local json = require("json")
local utils = require("utils")
local const = require("const")
local request_method = ngx.var.request_method
local pl_utils = require("pl.utils")
local shell = require("shell")

--utils.check_login()
--_, login_token = utils.check_permission("StreamCompute")

-- param formate: "2018-09-06 20:48:53.932 [MessageLoopThread-1536238026820] INFO Job - START SUCCESS "
local function split_datetime(param)
    if param == nil then
        return false
    end
    local date_fmt = "%d%d%d%d%-%d%d%-%d%d %d%d%:%d%d%:%d%d%.%d%d%d"
    local index = string.find(param, date_fmt)

    if index ~= nil then
        local timestamp = string.sub(param, string.find(param, date_fmt))
        local content = string.sub(param, #timestamp + 1)
        return true, timestamp, content
    end

    return false
end

function split_str(str)
    lines = {}
    for s in str:gmatch("[^\n]+") do
        table.insert(lines, s)
    end
    return lines
end

-- timeToConvert = "2011-01-01 01:30:33"
local function datetime_to_utc(timeToConvert)

    if timeToConvert == nil then
        return 0
    end
    local pattern = "(%d+)-(%d+)-(%d+) (%d+):(%d+):(%d+)"
    local runyear, runmonth, runday, runhour, runminute, runseconds = timeToConvert:match(pattern)
    local convertedTimestamp = os.time({year = runyear, month = runmonth, day = runday, hour = runhour, min = runminute, sec = runseconds})

    return convertedTimestamp
end

local function utc_to_datetime(utc)

    local tab = os.date("*t", utc)

    return ""..tab["year"].."-"..tab["month"].."-"..tab["day"].." "..tab["hour"]..":"..tab["min"]..":"..tab["sec"]
end

local function check_params(taskNo, start_time, end_time)

    if taskNo == nil or type(taskNo) ~= "string" then
        utils.error_exit(400, "task NO must not be null")
    end

    if (start_time ~= nil and end_time ~= nil) then
        if tonumber(end_time) <= tonumber(start_time) then
            utils.error_exit(400, "start time must > end time")
        end
    elseif(start_time == nil and end_time == nil) then
        print("start time ")
    else
        utils.error_exit(400, "start/end time is not matched")
    end
end

if "POST" == request_method then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_boby_data()
    local pageSize = params["PageSize"]
    local currentPage = params["CurrentPage"]
    local taskNO = params["TaskNO"]
    local startTime = params["StartTime"]
    local endTime = params["EndTime"]
    local searchKey = params["SearchKey"]

    if searchKey == nil then
        searchKey = " "
    elseif #searchKey == 0 then
        searchKey = " "
    end
    utils.check_current_page(currentPage)
    pageSize = utils.check_page_size_allow_zero(pageSize)

    check_params(taskNO, startTime, endTime)

    if startTime == nil and endTime == nil then
        startTime = tonumber(os.time() - 365*24*60*60)*1000
        endTime = tonumber(os.time())*1000
    end

    startTime = tonumber(startTime)/1000
    endTime = tonumber(endTime)/1000

    --find . -type f -newermt '2018-01-01 00:00:00' ! -newermt '2018-09-09 12:00:00' -exec grep "count:342" {} \; | awk '{i = 2; for (; i <= NF; i++){ printf "%s ",$(i)} print ""}'
    local cmd_fmt = "find %s -type f -newermt \"%s\" ! -newermt \"%s\" -exec grep \"%s\" {} \\; | awk '{i = 1; for (; i <= NF; i++){ printf \"%%s \",$(i)} print \"\"}'"

    local dbus_cmd = {}
    local file_path = nil
    dbus_cmd["id"] = taskNO
    local ok, msg_raw, err = utils.dbus_call_edge(const.stream_compute.wkn,
                                    const.stream_compute.obj,
                                    const.stream_compute.inf,
                                    const.stream_compute.method.getJobDetail,
                                    json.encode(dbus_cmd))

    local msg_ret = {}
    if (ok and msg_raw) then
        local ok, msg = pcall(json.decode, msg_raw[1])
        if (ok and 0 == msg["code"]) then
            file_path = msg["data"]["logPath"]
        else
            utils.error_exit(500, json.encode("failed to query: "..msg_raw[1]))
        end
    else
        utils.error_exit(500, json.encode("failed to query: "..err))
    end

    local cmd = string.format(cmd_fmt, file_path, utc_to_datetime(startTime), utc_to_datetime(endTime), searchKey)

    local res, actual, stdout, errout = pl_utils.executeex(cmd, false)

    local msg_ret = {}
    if res then
        local tab = split_str(stdout)
        local list_ret = {}
        if tonumber(pageSize) == 0 then
            pageSize = #tab
        end
        local min, max = utils.get_item_index(#tab, pageSize, currentPage)
        local item = {}
        local total_count = 0
        for k, v in ipairs(tab) do
            if k >= min and k <= max then
                local ret, timestamp, content = split_datetime(v)
                local cur_time = datetime_to_utc(timestamp)
		        if item["content"] ~= nil then
		            item["content"] = item["content"]..v
                    if split_datetime(tab[k + 1]) == true or k == #tab then
                        table.insert(list_ret, item)
                        item = {}
                    end
		        end
                if cur_time >= startTime and cur_time <= endTime then
                    item["timestamp"] = timestamp
                    item["content"] = content
                    if split_datetime(tab[k + 1]) == true or k == #tab then
                        table.insert(list_ret, item)
                        total_count = total_count + 1
                        item = {}
                    end
                else
                    max = max + 1
                end
           end
        end
        msg_ret["TotalNum"] = total_count 
        msg_ret["PageSize"] = pageSize
        msg_ret["CurrentPage"] = currentPage
        msg_ret["List"] = list_ret
        utils.error_exit(200, json.encode(msg_ret))
    else
        msg_ret["List"] = ""
        utils.error_exit(500, "failed to search log: "..errout)
    end
end
utils.error_exit(400, "unsupported request method")

