local json = require("json")
local utils = require("utils")
local const = require("const")
local pl_pretty = require("pl.pretty")

local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("UserManage")

if ("GET" == request_method) then

    local args = ngx.req.get_uri_args()
    page_size = tonumber(args["PageSize"])
    current_page = tonumber(args["CurrentPage"])

    utils.check_current_page(current_page)
    utils.check_page_size(page_size)


    local ok, msg = utils.get_data(const.data_key.eweb.user)
    if (ok and msg) then
        local user_list_array = {}
        local total_user = 0
        user_list = json.decode(msg)

        for k, v in pairs(user_list["UserList"]) do
            table.insert(user_list_array, k)
            total_user = total_user + 1
        end

        table.sort(user_list_array)
        res = {}
        res["TotalNum"] = total_user
        res["PageSize"] = page_size
        res["CurrentPage"] = current_page
        res["List"] = {}

        min, max = utils.get_item_index(total_user, page_size, current_page)
        for i = min, max do
            user = {}
            username = user_list_array[i]
            user_info = user_list["UserList"][username]
            user["Username"] = username
            user["Permission"] = user_info["Permission"]
            user["LastLoginTime"] = user_info["LastLoginTime"]
            table.insert(res["List"], user)
        end

        res_raw = json.encode(res)
        utils.error_exit(200, res_raw)
    else
        utils.error_exit(500, "get user list failed")
    end

end

utils.error_exit(400, "unsupported request method")
------ EOF ------
