#!/bin/bash

LINK_EDGE_HOME=/linkedge
DPS_HOME=$LINK_EDGE_HOME/gateway/build/bin/dps
DPS_DATA_ROOT=$LINK_EDGE_HOME/run
DPS_DATA=$LINK_EDGE_HOME/run/dps
DPS_SOCKET_PATH=/dev/socket
LD_SO_PRELOAD=/etc/ld.so.preload

##############################################################
# Append LD_PRELOAD to /etc/ld.so.preload
##############################################################
create_dps_preload()
{
  echo "create DPS preload..."

  LINKER_PREFIX="$DPS_HOME/bin/linker "

  if [ ! -f $LD_SO_PRELOAD ]; then
    echo "create $LD_SO_PRELOAD"
    touch $LD_SO_PRELOAD
  fi

  if [ ! -f $LD_SO_PRELOAD.link-iot-edge-backup ]; then
    cp -f $LD_SO_PRELOAD $LD_SO_PRELOAD.link-iot-edge-backup
  fi

  find_ld_preload=$(cat $LD_SO_PRELOAD | grep $LINKER_PREFIX)

  if [ ! -z "$find_ld_preload" ]; then
    echo "ld.so.preload already set"
  else
    echo "set linker to $LD_SO_PRELOAD"
    #echo_data=$(echo -n $DPS_HOME'/bin/linker '; cat $LD_SO_PRELOAD)
    #echo $echo_data > $LD_SO_PRELOAD.dps
    #mv $LD_SO_PRELOAD.dps $LD_SO_PRELOAD
    echo $DPS_HOME'/bin/linker ' >> $LD_SO_PRELOAD
  fi

  chmod 644 $LD_SO_PRELOAD
  echo "DPS preload created."
}

##############################################################
# Disable system auditd service
##############################################################
disable_system_audit()
{
  service auditd stop
  systemctl disable auditd
}

##############################################################
# Create essential directories for DPS
##############################################################
create_dps_dirs()
{
  mkdir -p $DPS_DATA/log
  mkdir -p $DPS_DATA/pattern
  mkdir -p $DPS_DATA/patch
  mkdir -p $DPS_DATA/policy
  mkdir -p $DPS_DATA/etc
  mkdir -p $DPS_SOCKET_PATH
  chmod +x $DPS_DATA_ROOT
}


##############################################################
# Apply SELinux policy in order to allow iptables
##############################################################
apply_dps_sepolicy()
{
  echo "Applying DPS SElinux policies..."
  echo "Set linker context"
  semanage fcontext -a -t file_context_t '/linkedge/gateway/build/bin/dps/bin/linker'
  restorecon -v '/linkedge/gateway/build/bin/dps/bin/linker'
  echo "Set DPS data context"
  semanage fcontext -a -t file_context_t '/linkedge/run/dps'
  restorecon -v '/linkedge/run/dps'
  echo "Set allow policies..."
  semodule -i /linkedge/gateway/build/bin/dps/script/dps-iptables-le.pp
}

create_dps_preload
disable_system_audit

SE_STATUS=$(getenforce)
if [ "$SE_STATUS" == "Enforcing" ]; then
  apply_dps_sepolicy
else
  echo "SELinux is not enforcing, ignore"
fi

create_dps_dirs

#cd `dirname $0`

cd $DPS_HOME/bin/
./dpsd &

